#!/bin/sh

STATE_DIR="/var/run/fmtapper"
CONF_DIR="/etc/fmtapper/profiles/$1"
CONF_FILE="$CONF_DIR/fmtapper.conf"
STATE_FILE="$STATE_DIR/$1.state"
PID_FILE="$STATE_DIR/$1.pid"
CMD="/usr/bin/fmtapper --conf $CONF_FILE"
SOCKET_FILE="/tmp/${1}.sock"
STATUS_FILE="$CONF_DIR/stopped"

do_log()
{
    NOW=`date +"%Y-%m-%d %H:%M:%S"`
    echo "[$NOW] $1"
}

do_start()
{
    KILLED=0
    do_log "Starting fmtapper"
    trap 'do_log "Killing fmtapper"; KILLED=1; kill -9 $PID' TERM INT

    mkdir -p $STATE_DIR
    echo 0 > $STATE_FILE

    if [ -f $STATUS_FILE ]; 
    then
        do_log "File '$STATUS_FILE' is found, it means that service was manually stopped previously."
        return 1
    fi

    if [ -s $PID_FILE ];
    then
        PID=`cat $PID_FILE`
        if test "$PID" -gt 0 ;
        then
            do_log "fmtapper already running with PID $PID, killing"
            kill $PID
        fi
    fi

    if [ "$(ntpq -p | grep '*' | wc -l)" -ne "1" ];
    then
      echo "Ntp service wasn't started!"
      return 2
    fi

    $CMD &
    PID=$!
    echo $PID > $PID_FILE
    do_log "fmtapper started with PID $PID"

    wait $PID
    trap - TERM INT
    wait $PID
    FMTAPPER_STATUS=$?
    do_log "fmtapper finished with $FMTAPPER_STATUS"
    if test $KILLED -eq 1
    then
	    EXIT_STATUS=0
    else
	    EXIT_STATUS=$FMTAPPER_STATUS
    fi

    return $EXIT_STATUS
}

do_exit()
{
    if test $1 -eq 0
    then
        do_log "Removing state file $STATE_FILE"
        rm $STATE_FILE
    else
        do_log "Writing exit status $1 to $STATE_FILE"
        echo $1 > $STATE_FILE
    fi

    if [ -s $PID_FILE ]
    then
        do_log "deleting pid file $PID_FILE"
        rm $PID_FILE
    fi

    if [ -e $SOCKET_FILE ]
    then
        do_log "deleting socket file $SOCKET_FILE"
        rm $SOCKET_FILE
    fi

    exit 0
}

while :
do
    do_start
    case $? in
	22)
	    do_log "Invalid configuration. Exiting."
	    do_exit 22;;
	111)
	    do_log "Connection refused. Exiting."
	    do_exit 111;;
	139)
	    do_log "No free space"
	    sleep 300
	    do_log "Restarting after delay";;
	1)
	    do_log "Start was interrupted."
	    exit 0;;
	0)
	    do_log "Finished successfully."
	    do_exit 0;;
	*)
	    sleep 30
	    do_log "Restarting after failure";;
    esac
done
