from flask import Flask, g, current_app
import utils
from frontend.views import mod as main_views
from metrics import MetricsThread
from frontend.utils import MyJsonEncoder
import wtforms_json
import services
import tuner_service_utils
import config_utils
from tuner import Tuner
import logging
import os


def create_app():
    app = Flask(__name__)
    if os.getenv('TAPPERUI_DEBUG', False):
        app.config.from_object('fmtapper_web.config.DevelopmentConfig')
    else:
        app.config.from_object('fmtapper_web.config.Config')

    app.config.from_envvar('FMTAPPER_WEB_SETTINGS', silent=True)
    app.config.setdefault('INSTALLATION_ID', services.get_installation_id(app.config['INSTALLATION_ID_FILE']))
    app.secret_key = '\xa0T\xb6\x8d\xfex\x1c\xa0TUE\xd0\xc6\xee\xff\x83fmmv\x8f\xd5[\xc5'

    if not app.debug:
        app.logger.setLevel(logging.DEBUG)
        del app.logger.handlers[:]
        handler = logging.StreamHandler()
        handler.setFormatter(logging.Formatter("[%(asctime)s] %(message)s", datefmt="%Y-%m-%d %H:%M:%S"))
        app.logger.addHandler(handler)

    init_db(app)

    def start_metrics():
        t = MetricsThread(app)
        t.start()

    def before_request():
        get_db(app.config['DATABASE'])

    app.before_request(before_request)
    app.add_template_filter(format_ram, "format_ram")
    app.teardown_appcontext(teardown)

    app.register_blueprint(main_views)
    app.before_first_request(start_metrics)
    app.before_first_request(check_tuner_path)
    app.before_first_request(recount_ports_in_use)
    app.json_encoder = MyJsonEncoder

#   init wtforms-json
    wtforms_json.init()
    print 'Created app'
    return app


def check_tuner_path():
    tuner_list = services.get_tuners()
    standart_url = current_app.config['STANDART_MP3_FILEPATH']
    for tuner in tuner_list:
        if tuner.encoding_config.url == standart_url:
            continue
        tuner.encoding_config.url = standart_url
        services.update_tuner(tuner)

def recount_ports_in_use():
    tuner_list = services.get_tuners()
    parser = config_utils.get_ports_config()
    ports_in_use = []
    for tuner in tuner_list:
        if tuner.device_type == Tuner.DeviceType.FM_TUNER:
            tuner_status, _, _, _ = tuner_service_utils.get_tuner_status(tuner.id)
            if tuner_status == Tuner.TunerStatus.RUNNING:
                ports_in_use.append(tuner.device_config.udp_port)
    ports_in_use = ','.join(ports_in_use)
    parser.set('ports-in-use', ports_in_use)
    parser.write_config()


def format_ram(v):
    return u'{0:.2f}'.format((1.0*v)/1024/1024/1024)


def get_db(database):
    db = getattr(g, '_database', None)
    if db is None:
        db = g._database = utils.connect_to_db(database)
    return db


def init_db(app):
    with app.app_context():
        db = get_db(app.config['DATABASE'])
        with app.open_resource('schema.sql', mode='r') as f:
            db.cursor().executescript(f.read())
        db.commit()


def teardown(exception):
    db = getattr(g, '_database', None)
    if db is not None:
        db.close()
