from ConfigParser import ConfigParser
from json_utils import JSONIterator
from config_utils import FakeGlobalSectionHead
import config_utils
import services
from fmtapper_web.errors import ConfigError
import os
from flask import current_app


class CleanupService(JSONIterator):
    def __init__(self, store_logs=None, tuner_logs_max_weight=None, audio_max_weight=None, audio_check_interval=None):
        self.store_logs = store_logs
        self.audio_max_weight = audio_max_weight
        self.audio_check_interval = audio_check_interval
        self.tuner_logs_max_weight = tuner_logs_max_weight

    def to_json(self):
        rv = dict()
        rv['store_logs'] = self.store_logs
        rv['audio_max_weight'] = self.audio_max_weight
        rv['audio_check_interval'] = self.audio_check_interval
        rv['tuner_logs_max_weight'] = self.tuner_logs_max_weight
        return rv

    def save_to_file(self):
        parser = ConfigParser()
        sec = 'global'
        parser.add_section(sec)
        parser.set(sec, 'store-logs', self.store_logs)
        parser.set(sec, 'audio-max-weight', self.audio_max_weight)
        parser.set(sec, 'audio-check-interval', self.audio_check_interval)
        parser.set(sec, 'tuner-logs-max-weight', self.tuner_logs_max_weight)

        for tuner in services.get_tuners():
            sec = tuner.id
            parser.add_section(sec)
            parser.set(sec, "store-audio", tuner.store_audio)
            parser.set(sec, "store-logs", tuner.store_logs)
            parser.set(sec, "audio-dir", tuner.get_audio_dir())
            parser.set(sec, "log-dir", tuner.log_dir)

        with open(config_utils.get_cleanup_service_config_path(), 'w') as fp:
            parser.write(FakeGlobalSectionHead(fp))

    @staticmethod
    def load_from_file():
        logger = current_app.logger
        file_path = config_utils.get_cleanup_service_config_path()

        if os.path.isfile(file_path):
            logger.debug('Reading cleanup service configuration from file %s', file_path)
        else:
            raise ConfigError('File %s not exists', file_path)

        parser = config_utils.get_parser(file_path)
        sec = 'global'
        store_logs = parser.getint(sec, 'store-logs')
        audio_max_weight = parser.getint(sec, 'audio-max-weight')
        audio_check_interval = parser.get(sec, 'audio-check-interval')
        tuner_logs_max_weight = parser.getint(sec, 'tuner-logs-max-weight')
        return CleanupService(store_logs=store_logs, audio_max_weight=audio_max_weight,
                                    audio_check_interval=audio_check_interval,
                                    tuner_logs_max_weight=tuner_logs_max_weight)
