# -*- coding: utf-8 -*-

from wtforms import ValidationError
import re
from urlparse import urlparse
from fmtapper_web.tuner import BroadcastConfig
import fmtapper_web.services as services

def validate_time_interval(form, field):
    if field.data is not None:
        m = re.search('\A(\d\d):(\d\d):(\d\d)\Z', field.data)
        if m:
            days = int(m.group(1))
            hours = int(m.group(2))
            minutes = int(m.group(3))
            if days < 0:
                raise ValidationError(u'Неверное значение дней в интервале времени')
            if hours > 23 or hours < 0:
                raise ValidationError(u'Неверное значение часов в интервале времени')
            if minutes > 59 or minutes < 0:
                raise ValidationError(u'Неверное значение минут в интервале времени')
            if hours == 0 and minutes == 0 and days == 0:
                raise ValidationError(u'Продолжительность интервала времени должна быть больше 0')

        else:
            raise ValidationError(u'Неверный формат интервала времени')


def is_not_empty(s):
    return bool(s and s.strip())


def validate_ftp_conf_field(form, field):
    if is_not_empty(form.ftp_url.data) and not is_not_empty(field.data):
        raise ValidationError(u'Обязательное поле.')


def validate_ftp_time_interval(form, field):
    if form.ftp_url.data != '':
        validate_time_interval(form, field)


def validate_latin(form, field):
    if field.data is not None:
        if not all(ord(c) < 128 for c in field.data):
            raise ValidationError(u'Допустимы только латинские символы.')

def validate_feed_url(form, field):
    if field.data is not None and form.server_type.data == str(BroadcastConfig.BroadscastType.FFSERVER):
        if field.data.strip() == '':
            return
        if '/feed' not in field.data:
            raise ValidationError(u'Неверный формат url.')

def validate_spaces(form, field):
    if field.data is not None:
        if len(re.findall('[^\S]+', field.data)) > 0:
            raise ValidationError(u'Содержит пробелы.')


def validate_channel_id(form, field):
    curr_tuner_id = form.id.data
    channel_id = field.data
    if channel_id is not None:
        tuner_list = services.get_tuners()
        tuner_list = list(filter(lambda tun: tun.id != curr_tuner_id, tuner_list))
        channels_ids = list(map(lambda tuner: tuner.channel_id, tuner_list))
        if channel_id in channels_ids:
            raise ValidationError(u'Указанный Id канала уже используется.')