# -*- coding: utf-8 -*-

from wtforms import Form, validators as v, StringField, IntegerField,  FormField, FieldList, ValidationError
from fmtapper_web.schedule import Interval
from fmtapper_web.frontend.forms import TapperUIBaseForm
import datetime


class IntervalForm(TapperUIBaseForm):
    start = StringField(u'Начало интервала', [v.number_range(min=0, max=86400)])
    end = StringField(u'Конец интервала', [v.number_range(min=0, max=86400)])

    def validate(self):
        form_valid = Form.validate(self)

        if not form_valid:
            return False
        start = int(self.start.data)
        end = int(self.end.data)
        if start >= end:
            start = datetime.timedelta(seconds=start)
            end = datetime.timedelta(seconds=end)
            raise ValidationError(u'Время начала должно быть меньше времени окончания: %s >= %s' % (start, end))
        return True


class ScheduleForm(TapperUIBaseForm):
    id = StringField(u'Название', [v.required()])
    intervals = FieldList(FormField(IntervalForm), min_entries=1)

    def validate(self):
        form_valid = Form.validate(self)
        if not form_valid:
            return False

        intervals = map(lambda i: Interval(**i), self.intervals.data)
        if Interval.is_overlapped_intervals(intervals):
            raise ValidationError(u'Интервалы не должны пересекаться')
        return True

