'use strict';

angular.module('fmtapper.controllers', ['restangular'])
.config(['RestangularProvider', '$locationProvider', function(RestangularProvider, $locationProvider) {
        RestangularProvider.setBaseUrl('/api');
        RestangularProvider.setDefaultHttpFields({ cache: false });
		RestangularProvider.setResponseExtractor(function(response, operation) {
		      var newResponse;
		      if (operation === "getList") {
                if(response.data === '')
                    newResponse = [];
                else
                    newResponse = response.data;
		        newResponse.metadata = {
		        		success: response.success,
		        		results: response.data ? response.data.length : 0,
		        		page: 1,
		        		totalPages: 1
		        	};
		      }
		      else {
		        newResponse = response;
		      }
		      return newResponse;
		    });
        RestangularProvider.addResponseInterceptor(function(resp) {
            if(resp.status && resp.status == 403) {
                window.location.href = '/login';
                return resp;
            }
            else {
                return resp;
            }
        });
		RestangularProvider.setErrorInterceptor(function(resp) {
	    	if(resp.status !== 200) {
                $.notify("Произошла неизвестная ошибка", {
                    placement: {
                        from: 'top',
                        align: 'center'
                    },
                    type: "danger"
                });
	    		return true;
	    	}
	    	return true;
	    });

        $locationProvider.html5Mode(true).hashPrefix('');
    }])
.controller('ListTunersCtrl', ['$scope', '$location', 'Restangular', '$routeParams', '$http', '$window', '$interval',
        function($scope, $location, Restangular, $routeParams, $http, $window, $interval) {
            $window.document.title = 'ТУК - тюнеры';
            $('#cleanup_link').removeClass('active');
            $('#tuners_link').addClass('active');

            var initChart = function(info) {
                    var data2series = function(data, name, label, color) {
                            var tzOffset = new Date().getTimezoneOffset() * 60,
                                values = _.map(data, function(val) {
                                    return [1000*(val['ts'] - tzOffset), val[name]];
                                }),
                                res = {
                                    data: values,
                                    label: label
                                };
                            if(color) {
                                res.color = color;
                            }
                            return res;
                        },
                        options = {
                        xaxis: {
                            mode: "time",
                            timeformat: "%d.%m %H:%M",
                        },
                        yaxis: {
                            max: 100,
                            min: 0,
                            tickFormatter: function(v) {
                                return v + ' %';
                            },
                            zoomRange: false,
                            panRange: false
                        },
                        legend: {
                            position: 'ne'
                        },
                        zoom: {
                            interactive: true
                        },
                        pan: {
                            interactive: true
                        }
                    };

                    var series = [data2series(info['metrics'], 'cpu', 'CPU', '#00ff00'),
                        data2series(info['metrics'], 'ram', 'RAM', '#0000ff')];
                    $.plot($('#chart'), series, options);
                };

            $scope.getMetrics = function() {
                $http.get('/api/metrics').success(function(resp) {
                    $scope.system_info = resp.data;
                    initChart(resp.data);
                });
            };

            var stopTuners;
            $scope.startUpdateTuners = function() {
                if(angular.isDefined(stopTuners)) {
                  return;
                }
                stopTuners = $interval(function() {
                    Restangular.all('tuners_status').getList().then(function(tuners) {
                        _.forEach($scope.tuners, function(tuner) {
                            var tuner_info = _.find(tuners, function(t) {
                                return t.tuner_id == tuner.id;
                            });
                            if(tuner_info) {
                                tuner.runtime_info = tuner_info;
                            }
                        })
                    });
                }, 5000);

            };

            var stopMetrics;
            $scope.startUpdateMetrics = function() {
                if(angular.isDefined(stopMetrics)) {
                    return;
                }
                stopMetrics = $interval($scope.getMetrics, 60000);
            };

            $scope.stopUpdate = function() {
                if(angular.isDefined(stopTuners)) {
                    $interval.cancel(stopTuners);
                    stopTuners = undefined;
                }

                if(angular.isDefined(stopMetrics)) {
                    $interval.cancel(stopMetrics);
                    stopMetrics = undefined;
                }
            };

            $scope.$on('$destroy', function() {
                $scope.stopUpdate();
            });

            $scope.tuners = Restangular.all('tuners').getList().$object;
            $scope.fmtapper_ver = Restangular.one('fmtapper_ver').get().$object;
            $scope.tapperui_ver = Restangular.one('tapperui_ver').get().$object;
            $scope.getMetrics();

            $scope.startUpdateTuners();
            $scope.startUpdateMetrics();

    }])
.controller('EditTunerCtrl', ['$scope', '$location', 'Restangular', '$routeParams', '$http', 'FormService',
        'NotificationService', '$sce', '$window', '$interval', 'IntervalService', function($scope, $location, Restangular, $routeParams,
        $http, FormService, NotificationService, $sce, $window, $interval, IntervalService) {
        $('#cleanup_link').removeClass('active');
        $('#tuners_link').addClass('active');

        $scope.fmtapper_features = Restangular.one('fmtapper_features').get().$object;
        $scope.fileMaskTail = '/{{channel}}/{{year}}_{{month}}_{{day}}/{{hour}}_{{minute}}_{{second}}.mp3';
        $scope.fileMaskRoot = '/home/support/media';

        $scope.loadTuner = function(id) {
            if(id) {
                $scope.tuner = Restangular.one('tuners', id).get().then(function(resp) {
                    $scope.tuner = resp;
                    if(resp.device_config && resp.device_config.name) {
                        var devices, device_names, device_config;
                        switch(resp.device_type) {
                            case 0:
                                devices = $scope.audio_devices;
                                device_names = $scope.audio_device_names;
                                device_config = {
                                    name: resp.device_config.name,
                                    sample_rate: resp.device_config.sample_rate
                                };
                                break;
                            case 1:
                                devices = $scope.usb_devices;
                                device_names = $scope.usb_device_names;
                                device_config = {
                                    name: resp.device_config.name,
                                    sample_rate: resp.device_config.sample_rate,
                                    frequency: resp.device_config.frequency
                                };
                                break;
                        }
                    }
                    if(device_names && !_.find(device_names, function(name){return name == resp.device_config.name;})) {
                        device_names.splice(0, 0, resp.device_config.name);
                        devices.splice(0, 0, device_config);
                    }

                    $scope.encodingChanged();
                    $scope.updateFtpStatus(resp.ftp_config.ftp_status);
                    $scope.updateTuner();
                    // start update state
                    $scope.startUpdateTuner();
                    if($scope.tuner.broadcast_config.feed_url != null &&
                        $scope.tuner.broadcast_config.server_type == 0 &&
                        $scope.tuner.broadcast_config.feed_url.trim()){
                        $scope.updateStreamUrl();
                    }
                    $scope.updateChannelNum();

                    var fileMaskTailPosition = $scope.tuner.encoding_config.url.indexOf('/{{');
                    $scope.fileMaskRoot = $scope.tuner.encoding_config.url.substring(0, fileMaskTailPosition);
                    if(fileMaskTailPosition != -1) {
                        var len = $scope.tuner.encoding_config.url.length;
                        $scope.fileMaskTail = $scope.tuner.encoding_config.url.substring(fileMaskTailPosition, len);
                    }
                });
            }
            else {
                $scope.tuner = {
                    device_type: 0,
                    encoding_config: {
                        bit_rate: 192000,
                        sample_rate: 44100,
                        mono: false
                    },
                    device_config: {},
                    broadcast_config: {
                        server_type: 2,
                        feed_url: ''
                    }
                };
                $scope.maskRootChanged();
                if($scope.audio_devices.length > 0) {
                    $scope.tuner.device_config.name = $scope.audio_devices[0].name;
                    $scope.tuner.device_config.sample_rate = $scope.audio_devices[0].sample_rate;
                }
                $scope.updatePortsConfig();
            }

            $scope.ffserverStreamUrls = [];
            for(var i = 1; i <= 6; i++) {
                $scope.ffserverStreamUrls.push('http://localhost:8090/feed' + i + '.ffm');
            }
        };

        $scope.updateStreamUrl = function() {
            if( $scope.tuner.broadcast_config.server_type == 0){
                var feed = $scope.tuner.broadcast_config.feed_url;
                var ind = feed.indexOf('/feed');
                if(ind == -1)
                    return
                var num = feed[ind + 5];
                $scope.tuner.broadcast_config.stream_url = window.location.origin + '/radio/stream' + num + '.mp3';
            }
            else {
                $scope.tuner.broadcast_config.stream_url = '';
                if($scope.tuner.broadcast_config.server_type == 2 ||
                    $scope.tuner.broadcast_config.server_type == 1)
                    $scope.tuner.broadcast_config.feed_url = '';
            }
        };


        $scope.updatePorts = function() {
            var port = 5000;
            var channel = parseInt($scope.tuner.broadcast_config.channel_on_tuner) - 1;
            $scope.tuner.device_config.port = port + channel;
        };

        $scope.updateChannelNum = function() {
            var num = $scope.tuner.device_config.port % 5000;
            if (num > 3)
                return
            $scope.tuner.broadcast_config.channel_on_tuner = num + 1;
        };

        $scope.maskRootChanged = function() {
            $scope.tuner.encoding_config.url = $scope.fileMaskRoot + $scope.fileMaskTail;
        };

        $scope.loadDevices = function() {
          Restangular.all('devices').getList().then(function(resp) {
              var devices = _.filter(resp, function(d) {
                  return !d.tuner || ($scope.tunerId && d.tuner == $scope.tunerId);
              });
              $scope.audio_devices = _.filter(devices, function(d) {return d.dev_type == 0;});
              $scope.audio_device_names = _.map($scope.audio_devices, function(d) {return d.name;});
              $scope.usb_devices = _.filter(devices, function(d) {return d.dev_type == 1;});
              $scope.usb_device_names = _.map($scope.usb_devices, function(d) {return d.name;});

              $scope.loadTuner($scope.tunerId);
          });
        };

        $scope.reset = function() {
            $scope.tunerId = $routeParams.tunerId;
            if($scope.tunerId) {
                $scope.configUrl = $sce.trustAsResourceUrl('/configs/' + $scope.tunerId);
                $window.document.title = 'ТУК - управление тюнером ' + $scope.tunerId;
            }
            else {
                $window.document.title = 'ТУК - создать тюнер';

            }

            $scope.loadDevices($scope.tunerId);
        };

        var changeDevice = function(devices) {
            var device = _.find(devices, function(d) {
                return d.name == $scope.tuner.device_config.name;
            });
            if(device) {
                $scope.tuner.device_config.sample_rate = device.sample_rate;
            }
        };

        $scope.changeUsbDevice = function() {
            changeDevice($scope.usb_devices);
        };

        $scope.changeAudioDevice = function() {
            changeDevice($scope.audio_devices);
        };

        $scope.changeDeviceType = function() {
            $scope.tuner.device_config = {};
            switch($scope.tuner.device_type) {
                case 0:
                    if($scope.audio_device_names.size > 0) {
                        $scope.tuner.device_config.name = $scope.audio_device_names[0];
                    }
                    $scope.changeAudioDevice();
                    break;
                case 1:
                    if($scope.usb_device_names.size > 0) {
                        $scope.tuner.device_config.name = $scope.usb_device_names[0];
                    }
                    $scope.changeUsbDevice();
                    break;
            }
        };

        $scope.isEndlessRecord = false;
        $scope.encodingChanged = function() {
            if($scope.tuner.encoding_config.interval_length > 86399){
                var err = {'field_errors':{'interval_length':['Допустимы значения от 0 до 86399']}};
                FormService.emitServerErrors($scope, 'tunerForm', err, 'tuner.encoding_config');
            }
            else{
                FormService.clearServerErrors($scope, 'tunerForm', 'tuner.encoding_config.interval_length');
            }

            if($scope.tuner.encoding_config.fragment_length > 86399 || parseInt($scope.tuner.encoding_config.fragment_length) <= 0){
                var err = {'field_errors':{'fragment_length':['Допустимы значения от 1 до 86399']}};
                FormService.emitServerErrors($scope, 'tunerForm', err, 'tuner.encoding_config');
            }
            else {
                FormService.clearServerErrors($scope, 'tunerForm', 'tuner.encoding_config.fragment_length');
            }
        };

        $scope.onUploadIntervalChange = function () {
            $scope.tuner.ftp_config.ftp_upload_interval  = IntervalService.format($scope.tuner.ftp_config.ftp_upload_interval);
        }

        var errorHandler = function() {
                $scope.inProgress = false;
            },
            showErrors = function(resp) {
              FormService.emitServerErrors($scope, 'tunerForm', resp, 'tuner');
            },
            saveHandler = function(resp) {
                $scope.inProgress = false;
                if(resp.status && resp.status != 200) {
                    NotificationService.notify(resp.global_errors, 'danger');
                    try {
                        showErrors(resp);
                    } catch (e) {

                    }
                }
                else {
                    $scope.tuner = Restangular.restangularizeElement(null, resp.data, '/tuners');
                    NotificationService.notify('Профиль успешно создан', 'success');
                    $location.path('/tuners');
                }
            },
            updateHandler = function(resp) {
                $scope.inProgress = false;
                if(resp.status && resp.status != 200) {
                    showErrors(resp);
                }
                else {
                    $scope.tuner = Restangular.restangularizeElement(null, resp.data, '/tuners');
                    $scope.updateChannelNum();
                    $scope.updateFtpStatus($scope.tuner.ftp_config.ftp_status);
                    NotificationService.notify('Успешно сохранено', 'success');
                }
            },
            applyHandler = function(resp) {
                $scope.inProgress = false;
                if(resp.status && resp.status != 200) {
                    showErrors(resp);
                }
                else {
                    $scope.tuner = Restangular.restangularizeElement(null, resp.data, '/tuners');
                    $scope.updateChannelNum();
                    NotificationService.notify('Настройки успешно применены', 'success');
                    $scope.isBroadcastChanged = false;
                }
            },
            deleteHandler = function(resp) {
                if(resp && resp.status && resp.status != 200) {
                    NotificationService.notify('Произошла ошибка: ' + resp.global_errors, 'danger');
                }
                else {
                    $scope.stopUpdate();
                    NotificationService.notify('Профиль тюнера удален', 'success');
                    $location.path('/tuners');
                }
            },
            startStopHandler = function(resp) {
                $scope.inProgress = false;
                if(resp.status && resp.status != 200) {
                    NotificationService.notify(resp.global_errors, 'danger');
                }
                else {
                    $scope.reset();
                }
            };

        $scope.saveTuner = function() {
            FormService.clearServerErrors($scope, 'tunerForm', 'tuner');
            $scope.inProgress = true;
            if($scope.tunerId) {
                $scope.tuner.put().then(updateHandler, errorHandler);
            }
            else {
                Restangular.all('/tuners').post($scope.tuner).then(saveHandler, errorHandler);
            }
        };

        $scope.applyTuner = function() {
            FormService.clearServerErrors($scope, 'tunerForm', 'tuner');
            $scope.inProgress = true;
            if($scope.tunerId) {
                $scope.tuner.post().then(applyHandler, errorHandler);
            }
        };

        $scope.deleteTuner = function() {
            bootbox.prompt({
                title: 'Введите пароль',
                size: 'small',
                message: "Введите пароль для подтверждения удаления тюнера",
                inputType: 'password',
                buttons: {
                    confirm: {
                        label: 'Ok',
                        className: 'btn-primary',
                        callback: function() { }
                    },
                    cancel: {
                        label: 'Отмена',
                        callback: function() {}
                    }
                },
                callback: function(pwd) {
                            if (pwd !== null && pwd.trim().length > 0) {
                                $scope.tuner.remove({password: pwd}).then(deleteHandler, errorHandler);
                            }
                        }
            });
        };

        $scope.startTuner = function() {
            $scope.inProgress = true;
            $scope.tuner.all('started').post({}).then(startStopHandler, errorHandler);
        };

        $scope.stopTuner = function() {
            $scope.inProgress = true;
            $scope.tuner.one('started', '').remove().then(startStopHandler, errorHandler);
        };

        $scope.ftpStatusStyle = function(status) {
            if(status.indexOf('[info]') > -1)
               return 'text-success';
            if(status.indexOf('[warning]') > -1)
               return 'text-warning';
            return 'text-danger';
        };

        $scope.updateFtpStatus = function(status) {
            if(status != ''){
                    $scope.tuner.ftp_config.ftp_status_style = $scope.ftpStatusStyle(status)
                if(status.indexOf(']\t') > -1){
                    $scope.tuner.ftp_config.ftp_status = status.split('\t')[1];
                    return;
                }
                $scope.tuner.ftp_config.ftp_status = status;
                return;
            }
        };

        $scope.isSchedule = function(){
            if($scope.isAdmin){
                try{
                    return $scope.tuner.is_schedules_empty;
                }
                catch(err){}
            }
            return false;
        };

        $scope.startBroadcast = function() {
            $scope.inProgress = true;
            $scope.tuner.all('broadcasting').post({}).then(startStopHandler, errorHandler);
        };

        $scope.stopBroadcast = function() {
            $scope.inProgress = true;
            $scope.tuner.one('broadcasting', '').remove().then(startStopHandler, errorHandler);
        };

        $scope.updateTuner = function() {
            Restangular.one('tuners_status', $scope.tunerId).get().then(function(resp) {
                $scope.tuner.runtime_info = resp.data;
                $scope.updateFtpStatus(resp.data.ftp_status);
            });
        };

        var stop;
        $scope.startUpdateTuner = function() {
            if(!$scope.tunerId || angular.isDefined(stop)) {
              return;
            }
            stop = $interval($scope.updateTuner, 5000);
        };

        $scope.stopUpdate = function() {
            if(angular.isDefined(stop)) {
                $interval.cancel(stop);
                stop = undefined;
            }
        };

        $scope.archiveFile = null;
        $scope.restoreTuner = function(){
            if($scope.archiveFile != null){
                var fd = new FormData();
                fd.append('archive', $scope.archiveFile);
                Restangular.one('configs', $scope.tunerId)
                .withHttpConfig({transformRequest: angular.identity})
                .customPOST(fd, '', undefined, {'Content-Type': undefined})
                .then(function(resp) {
                        if(resp.status && resp.status != 200) {
                            NotificationService.notify(resp.global_errors, 'danger');
                        }
                        else{
                            location.reload();
                        }
                });
            }
            else{
                NotificationService.notify('Не выбран файл конфигурации', 'danger');
            }

        };

        $scope.updatePortsConfig = function() {
            Restangular.one('get-ports-config', '').get().then(function(resp) {
                var portsConfig = resp.data;
                var ports_range = portsConfig['udp_port_max'] - portsConfig['udp_port_min'] + 1;
                var ports_available = ports_range > portsConfig['ports_in_use'];
                $scope.tuner.runtime_info = {
                    ports_available: ports_available
                };
            });
        };

        $scope.addTunerFromArchive = function() {
                if($scope.archiveFile != null){
                    var fd = new FormData();
                    fd.append('archive', $scope.archiveFile);
                    Restangular.one('configs', 'add_new_tuner')
                    .withHttpConfig({transformRequest: angular.identity})
                    .customPOST(fd, '', undefined, {'Content-Type': undefined})
                    .then(function(resp) {
                        if(resp.status && resp.status == 406){
                            NotificationService.notify(resp.global_errors, 'danger');
                        }
                        else{
                            $scope.tuner = resp;
                            $scope.tuner.copy_schedules = true;
                            $scope.updatePortsConfig();
                            $scope.updateChannelNum();
                            NotificationService.notify('Настройки из файла конфигурации добавлены', 'success');
                        }
                    });
                }
                else{
                    NotificationService.notify('Не выбран файл конфигурации', 'danger');
                }
        };

        $scope.reset();

        $scope.isBroadcastChanged = false;
        $scope.$watch('tuner.broadcast_config', function(newVal, oldVal){
            if(oldVal == undefined && newVal != undefined){
                $scope.isBroadcastChanged = false;
            }
            else {
                $scope.isBroadcastChanged = true;
            }
        }, true);
    }])
.controller('ListSchedulesCtrl', ['$scope', '$location', 'Restangular', '$routeParams', '$http', 'NotificationService',
        '$window', function($scope, $location, Restangular, $routeParams, $http, NotificationService, $window) {
            $scope.tunerId = $routeParams.tunerId;
            $window.document.title = 'ТУК - расписания тюнера ' + $scope.tunerId;
            $('#cleanup_link').removeClass('active');
            $('#tuners_link').addClass('active');

            $scope.days = [
                {label: 'Понедельник', name: 'monday'},
                {label: 'Вторник', name: 'tuesday'},
                {label: 'Среда', name: 'wednesday'},
                {label: 'Четверг', name: 'thursday'},
                {label: 'Пятница', name: 'friday'},
                {label: 'Суббота', name: 'saturday'},
                {label: 'Воскресенье', name: 'sunday'}
            ];

            $scope.is_applied = true;
            $scope.getSchedules = function() {
                Restangular.one('tuners', $scope.tunerId).one('schedules').get().then(function(resp) {
                    $scope.schedules = resp.data['schedules'];
                    $scope.is_applied = resp.data['schedules'].is_applied;
                    $scope.week_schedules = {};
                    if(resp.data['schedules'].week_sch) {
                        _.forEach(resp.data['schedules'].week_sch, function (s) {
                            $scope.week_schedules[s] = true;
                        });
                    }
                });
            };

            $scope.createSchedule = function(name) {
                $location.path('/tuners/' + $scope.tunerId + '/schedules/edit').search('name', name);
            };

            $scope.deleteSchedule = function(name) {
                bootbox.confirm({
                    title: 'Удаление расписания',
                    message: "Удалить расписание " + name + '?',
                    buttons: {
                        confirm: {
                            label: 'Ok',
                            className: 'btn-primary',
                            callback: function() { }
                        },
                        cancel: {
                            label: 'Отмена',
                            callback: function() {}
                        }
                    },
                    callback: function(res) {
                        if (res) {
                            Restangular.one('tuners', $scope.tunerId).one('/schedules', name).remove().then(function(resp) {
                                if(resp && resp.status && resp.status != 200) {
                                    NotificationService.notify('Произошла ошибка сервера', 'danger');
                                }
                                else {
                                    NotificationService.notify('Расписание удалено', 'success');
                                    $location.path('/tuners/' + $scope.tunerId + '/schedules');
                                    $scope.is_applied = resp.data['is_applied'];
                                    if(!resp.data['is_applied']) {
                                        $scope.restartTunerDialog();
                                    }
                                }
                            },
                            function() {
                                NotificationService.notify('Произошла ошибка сервера', 'danger');
                            });
                        }
                    }
                });
            };

            $scope.restartTunerDialog = function(){
                bootbox.confirm({
                    title: 'Требуется перезагрузка',
                    message: 'Перезагрузить тюнер?',
                    buttons: {
                        confirm: {
                            label: 'Ok',
                            className: 'btn-warning',
                            callback: function() { }
                        },
                        cancel: {
                            label: 'Отмена',
                            callback: function() {}
                        }
                    },
                    callback: function(res) {
                        if(res){
                            $scope.restartTuner();
                        }
                        else{
                            $location.path('/tuners/' + $scope.tunerId + '/schedules');
                        }
                    }
                });
            };

            $scope.restartTuner = function(){
                Restangular.one('tuners', $scope.tunerId).one('restart').post().then(function(resp) {
                    if(resp.status && resp.status != 200) {
                        NotificationService.notify('Не удалось перезагрузить тюнер', 'danger');
                        $location.path('/tuners/' + $scope.tunerId + '/schedules');
                    }
                    else{
                        NotificationService.notify('Тюнер успешно перезагружен', 'success');
                        $location.path('/tuners/' + $scope.tunerId + '/schedules');
                    }
                });
            };

            $scope.getSchedules();
        }])
.controller('EditScheduleCtrl', ['$scope', '$location', 'Restangular', '$routeParams', 'FormService', 'NotificationService',
        '$window', function($scope, $location, Restangular, $routeParams, FormService, NotificationService, $window) {
            $scope.tunerId = $routeParams.tunerId;
            $window.document.title = 'ТУК - управление расписанием';
            $('#cleanup_link').removeClass('active');
            $('#tuners_link').addClass('active');

            $scope.dateOptions = {
                    dateFormat: 'yy-mm-dd'
            };

            $scope.reset = function() {
                var scheduleName = $routeParams.scheduleName;
                $scope.scheduleName = scheduleName;
                if(scheduleName) {
                    $scope.schedule = Restangular.one('tuners', $scope.tunerId).one('schedules', scheduleName)
                        .get().$object;
                }
                else {
                    $scope.scheduleName = $location.search()['name'];
                    $scope.isNew = true;
                    $scope.schedule = {
                        id: $scope.scheduleName,
                        intervals: []
                    };
                }
            };

            $scope.applySchedule = function() {
                FormService.clearServerErrors($scope, 'scheduleForm', 'schedule');
                $scope.inProgress = true;
                if($scope.tunerId) {
                    $scope.schedule.post().then(applyHandler, errorHandler);
                }
            };

            var errorHandler = function() {
                $scope.inProgress = false;
            },
            showErrors = function(resp) {
              FormService.emitServerErrors($scope, 'scheduleForm', resp, 'schedule');
            },
            saveHandler = function(resp) {
                $scope.inProgress = false;
                if(resp.status && resp.status != 200) {
                    showErrors(resp);
                }
                else {
                    $scope.schedule = Restangular.restangularizeElement(null, resp.data,
                        '/tuners/' + $scope.tunerId + '/schedules');
                    $scope.scheduleName = $scope.schedule.id;
                    NotificationService.notify('Расписание успешно создано', 'success');
                    $scope.isNew = false;
                }
            },
            updateHandler = function(resp) {
                $scope.inProgress = false;
                if(resp.status && resp.status != 200) {
                    showErrors(resp);
                }
                else {
                    $scope.schedule = Restangular.restangularizeElement(null, resp.data,
                        '/tuners/' + $scope.tunerId + '/schedules');
                    NotificationService.notify('Успешно сохранено', 'success');
                }
            },
            applyHandler = function(resp) {
                $scope.inProgress = false;
                if(resp.status && resp.status != 200) {
                    showErrors(resp);
                }
                else {
                    $scope.schedule = Restangular.restangularizeElement(null, resp, '/tuners/' + $scope.tunerId + '/schedules');
                    NotificationService.notify('Настройки успешно применены', 'success');
                }
            },
            deleteHandler = function(resp) {
                if(resp && resp.status && resp.status != 200) {

                }
                else {
                    NotificationService.notify('Расписание удалено', 'success');
                    $location.path('/tuners/' + $scope.tunerId + '/schedules');
                }
            };

            $scope.addInterval = function() {
                $scope.schedule.intervals.push({start: 0, end: 120});
            };

            $scope.deleteInterval = function(ind) {
                $scope.schedule.intervals.splice(ind, 1);
            };

            $scope.saveSchedule = function() {
                FormService.clearServerErrors($scope, 'scheduleForm', 'schedule');
                $scope.inProgress = true;
                if(!$scope.isNew) {
                    $scope.schedule.put().then(updateHandler, errorHandler);
                }
                else {
                    Restangular.one('tuners', $scope.tunerId).all('/schedules').post($scope.schedule).then(saveHandler, errorHandler);
                }
            };

            $scope.deleteSchedule = function() {
                bootbox.confirm({
                    title: 'Удаление расписание ',
                    message: "Удалить расписание " + $scope.schedule.id + '?',
                    buttons: {
                        confirm: {
                            label: 'Ok',
                            className: 'btn-primary',
                            callback: function() { }
                        },
                        cancel: {
                            label: 'Отмена',
                            callback: function() {}
                        }
                    },
                    callback: function(res) {
                    if (res) {
                        $scope.schedule.remove().then(deleteHandler, errorHandler);
                    }
                }
                });
            };

            $scope.reset();
        }])
.controller('ListAudioCtrl', ['$scope', '$location', 'Restangular', '$routeParams', '$http', 'NotificationService',
        '$sce', 'FileService', '$window',
        function($scope, $location, Restangular, $routeParams, $http, NotificationService, $sce, FileService, $window) {
            $scope.tunerId = $routeParams.tunerId;
            $scope.currentFile = '';
            $window.document.title = 'ТУК - аудио-файлы тюнера ' + $scope.tunerId;
            $('#cleanup_link').removeClass('active');
            $('#tuners_link').addClass('active');

            $scope.listDir = function(path) {
                $scope.parent = FileService.getParentDir(path);
                $scope.currentPath = path;
                $scope.fileItems = FileService.listDir(path, $scope.audioDir);
                $scope.fileItems = $scope.fileItems.filter(fileItem => !fileItem.path.endsWith('.json'));
                $window.scrollTo(0, 0);
            };

            $scope.downloadArchive = function() {
                bootbox.confirm({
                    message: "Загрузить архив с выбранными файлами?",
                    buttons: {
                        confirm: {
                            label: 'Да'
                        },
                        cancel: {
                            label: 'Нет'
                        }
                    },
                    callback: function (result) {
                        if(!result)
                            return;

                        var dialog = bootbox.dialog({
                            title: 'Ожидание загрузки',
                            message: '<p>' +
                                        '<i class="glyphicon glyphicon-refresh glyphicon-refresh-animate"></i> ' +
                                        'Идет формирование архива...' +
                                     '</p>',
                            closeButton: false
                        });

                        var checkedFiles = $scope.fileItems.filter(file => file.checked);
                        Restangular.one('/get-audio-archive/',  $scope.tunerId)
                            .withHttpConfig({responseType: 'blob'})
                            .post('', checkedFiles)
                            .then(
                                function(archResponse){
                                        dialog.modal('hide');
                                        var url = window.URL.createObjectURL(archResponse);
                                        var a = document.createElement("a");
                                        document.body.appendChild(a);
                                        a.style = "display: none";
                                        a.href = url;
                                        a.download = $scope.tunerId+'_audio.zip';
                                        a.click();
                                        window.URL.revokeObjectURL(url);
                                },
                                function(){
                                    dialog.modal('hide');
                                }
                            );
                    }
                });
            };

            $scope.checkAll = function() {
                _.each($scope.fileItems, function (file) {
                    file.checked = $scope.checkAllInput;
                })
                $scope.downloadActive = $scope.checkAllInput;
            };

            $scope.fileChecked = function() {
                $scope.downloadActive =  $scope.fileItems.some(file => file.checked);
            };

            Restangular.one('tuners', $scope.tunerId).one('audio', '').get().then(function(resp) {
                $scope.audioDir = resp.data;
                FileService.updateFileLinks($scope.audioDir, function(path) {
                    return '/get-audio/' + $scope.tunerId + '/' + path;
                });
                $scope.listDir($scope.audioDir.path);
            });

            $scope.play = function(path) {
                $scope.currentFile = path;
            };
        }])
.controller('ListLogsCtrl', ['$scope', '$location', 'Restangular', '$routeParams', '$http', 'NotificationService',
        'FileService', '$window', function($scope, $location, Restangular, $routeParams, $http, NotificationService,
                                              FileService, $window) {
            $scope.tunerId = $routeParams.tunerId;
            $window.document.title = 'ТУК - логи тюнера ' + $scope.tunerId;
            $('#cleanup_link').removeClass('active');
            $('#tuners_link').addClass('active');

            $scope.listDir = function(path) {
                $scope.parent = FileService.getParentDir(path);
                $scope.currentPath = path;
                $scope.fileItems = FileService.listDir(path, $scope.logsDir);
                $window.scrollTo(0, 0);
            };

            Restangular.one('tuners', $scope.tunerId).one('logs', '').get().then(function(resp) {
                $scope.logsDir = resp.data;
                $scope.listDir($scope.logsDir.path);
            });
        }])
.controller('EditCleanupServiceCtrl', ['$scope', '$location', 'Restangular', '$routeParams', 'FormService', 'NotificationService',
        '$window', 'IntervalService',function($scope, $location, Restangular, $routeParams, FormService, NotificationService, $window, IntervalService) {
            $window.document.title = 'ТУК - сервис очистки';

            $('#cleanup_link').addClass('active');
            $('#tuners_link').removeClass('active');

            var errorHandler = function() {
                $scope.inProgress = false;
            },
            showErrors = function(resp) {
              FormService.emitServerErrors($scope, 'editForm', resp, 'cleanupConfig');
            },
            updateHandler = function(resp) {
                $scope.inProgress = false;
                if(resp.status && resp.status != 200) {
                    showErrors(resp);
                }
                else {
                    $scope.cleanupConfig = Restangular.restangularizeElement(null, resp.data,
                        '/cleanup_service');
                    NotificationService.notify('Успешно сохранено', 'success');
                }
            };

            $scope.onAudioCheckIntervalChange = function () {
                $scope.cleanupConfig.audio_check_interval  = IntervalService.format($scope.cleanupConfig.audio_check_interval);
            };

            $scope.reset = function() {
                Restangular.one('/cleanup_service', '').get().then(function(resp) {
                    $scope.cleanupConfig = resp;
                });
            };

            $scope.saveSettings = function() {
                $scope.inProgress = true;
                FormService.clearServerErrors($scope, 'editForm', 'cleanupConfig');
                $scope.cleanupConfig.put().then(updateHandler, errorHandler);
            };

            $scope.reset();
        }])
.controller('ListCleanupServiceLogsCtrl', ['$scope', '$location', 'Restangular', '$routeParams', '$http', 'NotificationService',
        'FileService', '$window', function($scope, $location, Restangular, $routeParams, $http, NotificationService,
                                              FileService, $window) {
            $scope.tunerId = $routeParams.tunerId;
            $window.document.title = 'ТУК - логи сервиса очистки';

            $scope.listDir = function(path) {
                $scope.parent = FileService.getParentDir(path);
                $scope.currentPath = path;
                $scope.fileItems = FileService.listDir(path, $scope.logsDir);
            };

            Restangular.one('cleanup_service', '').one('logs', '').get().then(function(resp) {
                $scope.logsDir = resp.data;
                $scope.listDir($scope.logsDir.path);
            });
        }])
.controller('EditServiceCtrl', ['$scope', '$location', 'Restangular', '$routeParams', '$http', 'NotificationService',
        'FormService', '$window', function($scope, $location, Restangular, $routeParams, $http, NotificationService,
                                              FormService, $window) {
            $scope.tunerId = $routeParams.tunerId;
            $window.document.title = 'ТУК - сервис';
            $('#cleanup_link').addClass('active');
            $('#tuners_link').removeClass('active');
            // $scope.interfaceConfig = {};
            Restangular.one('get-network-interfaces', '').get().then(function(resp) {
                $scope.interfacesConfigs = resp.data.interfaces;
                $scope.interfaceConfig = resp.data.interfaces[0];
                $scope.is_changes_applied = resp.data.is_changes_applied;
            });

            Restangular.one('get-ports-config', '').get().then(function(resp) {
                $scope.portsConfig = resp.data;
            });

            $scope.isIntervalSmallerThanUsed = function() {
                var portsInterval = $scope.portsConfig.udp_port_max - $scope.portsConfig.udp_port_min + 1;
                $scope.portsConfig.portsInterval = portsInterval;
                return portsInterval < $scope.portsConfig.ports_in_use;
            };

            $scope.saveNetworkSettings = function() {
                if (!confirm('Сохранить настройки сети?'))
                    return
                $scope.inProgress = true;
                FormService.clearServerErrors($scope, 'editForm', 'hostConfig');
                Restangular.one('save-network-interface', '').post('', $scope.interfaceConfig).then(function(resp) {
                    $scope.inProgress = false;
                    if(resp.status && resp.status != 200) {
                        FormService.emitServerErrors($scope, 'editForm', resp, 'hostConfig');
                    }
                    else {
                        $scope.interfaceConfig = resp.data.adapter;
                        $scope.is_changes_applied = resp.data.is_changes_applied;
                        NotificationService.notify('Успешно сохранено', 'success');
                    }
                });
            };

            $scope.savePortsSettings = function() {
                if (!confirm('Сохранить настройки портов?'))
                    return

                Restangular.one('update-ports-config', '').post('', $scope.portsConfig).then(function(resp) {
                    $scope.inProgress = false;
                    if(resp.status && resp.status != 200) {
                        NotificationService.notify('Не удалось сохранить', 'danger');
                    }
                    else {
                        $scope.portsConfig = resp.data;
                        NotificationService.notify('Успешно сохранено', 'success');
                    }
                });
            };

            $scope.rebootDevice = function() {
                if (!confirm('Перезагрузить устройство?'))
                    return
                $scope.inProgress = true;
                Restangular.one('reboot-device', '').get().then(function(resp) {
                    $scope.inProgress = false;
                    $scope.is_changes_applied = true;
                });
            };
        }])
.controller('ClockCtrl', ['$scope', 'Restangular', '$interval', function($scope, Restangular, $interval) {
            var updateTimeInfo = function () {
                Restangular.one('current_server_time', '').get().then(function(resp) {
                    $scope.clock = parseInt(resp.data) * 1000;
                });
            }
            var updateNtpInfo = function () {
                Restangular.one('ntp-status', '').get().then(function(resp) {
                    $scope.is_ntp_active = resp.data['is_ntp_active'];
                });
            }

            updateTimeInfo();
            updateNtpInfo();

            $interval(function () {$scope.clock += 1000;}, 1000);

            $interval(function () {
                updateTimeInfo();
            }, 5000);
        }])
